<?php

    declare(strict_types=1);
    
    /**
     * @author: Daniel Schmitzer (daschmi@daschmi.de)
     * @date: 09.06.22
     * @time: 10:17
     */
     
    namespace wpsg;
	
	use phpseclib\File\ASN1\Element;

    class wpsg_invoice extends \wpsg_model {
		
		 static $_tableName = 'WPSG_TBL_RECHNUNGEN';
		
		 /**
		 * Lädt die Daten  
		 */
		public function load($invoice_id) {
		
			parent::load($invoice_id);
				
			$this->data = $this->db->fetchRow("SELECT I.* FROM `".WPSG_TBL_RECHNUNGEN."` AS I WHERE I.`id` = '".wpsg_q($invoice_id)."' ");
			
			if ($this->data === null || $this->data['id'] != $invoice_id) throw new \wpsg\Exception(
				wpsg_translate(__('Die Daten einer Rechnung konnten nicht geladen werden', 'wpsg'), $invoice_id)
			);
			
		}

        public function getOrderIds() {

            return array_map('intval', $this->db->fetchAssocField("
                SELECT `order_id` FROM `".WPSG_TBL_ORDER_INVOICE."` WHERE `invoice_id` = '".intval($this->getId())."'
            "));

        }

        public function setOrder(int $order_id) {

            // Könnte man über Uniqe der Datenbank überlassen, aber lieber so
            $exist = intval($this->db->fetchOne("SELECT `id` FROM `".WPSG_TBL_ORDER_INVOICE."` WHERE `order_id` = '".intval($order_id)."' AND `invoice_id` = '".intval($this->getId())."' "));

            if ($exist <= 0) {

                $this->db->ImportQuery(WPSG_TBL_ORDER_INVOICE, [
                    'order_id' => M1::q($order_id),
                    'invoice_id' => M1::q($this->getId())
                ]);

            }

        }

		public function isInvoice() {
			
			return $this->__get('rnr') !== '';
			
		}
		
		public function isStorno() {
			
			return $this->__get('gnr') !== '';
			
		}
		
		public function isMultiInvoice() {
			
			return $this->getMeta('version') === '3';
			
		}

		/**
		 * @throws \Exception
		 */
		public function getFilePath() {

            /*
					if (file_exists($this->getFilePath($order_id).$r_id.'.pdf')) { $file = $this->getFilePath($order_id).$r_id.'.pdf'; }
					else if (file_exists($this->getFilePath($order_id).$rnr.'.pdf')) { $file = $this->getFilePath($order_id).$rnr.'.pdf'; }
					else if (file_exists($this->getFilePath($order_id).'R'.$r_id.'.pdf')) { $file = $this->getFilePath($order_id).'R'.$r_id.'.pdf'; }
					else { $file = $this->getFilePath($order_id).'R'.$rnr.'.pdf'; }
             */

            $ym = date('Y/m', strtotime($this->__get('datum')));

			if ($this->isMultiInvoice()) {

				if ($this->shop->isMultiBlog()) $path = WP_CONTENT_DIR.'/'.WPSG_MB_UPLOADS.'/wpsg/wpsg_rechnungen/'.$ym.'/multi/'.$this->getId();
                else $path = WPSG_PATH_CONTENT.'uploads/wpsg/wpsg_rechnungen/'.$ym.'/multi/'.$this->getId();

			} else {

                if ($this->shop->isMultiBlog()) $path = WP_CONTENT_DIR.'/'.WPSG_MB_UPLOADS.'/wpsg/wpsg_rechnungen/'.$ym.'/';
                else $path = WPSG_PATH_CONTENT.'uploads/wpsg/wpsg_rechnungen/'.$ym.'/';

                if (!file_exists($path.$this->getId().'.pdf')) {

                    if ($this->__get('gnr') !== '') {

                        if (file_exists($path.'G'.$this->__get('gnr').'.pdf')) $path .= 'G'.$this->__get('gnr').'.pdf';
                        else if (file_exists($path.'G'.$this->getId().'.pdf')) $path .= 'G'.$this->getId().'.pdf';
                        else if (file_exists($path.$this->__get('gnr').'.pdf')) $path .= $this->__get('gnr').'.pdf';
                        else $path .= $this->getId().'.pdf';

                    } else if ($this->__get('rnr') !== '') {

                        if (file_exists($path.'R'.$this->__get('rnr').'.pdf')) $path .= 'R'.$this->__get('rnr').'.pdf';
                        else if (file_exists($path.'R'.$this->getId().'.pdf')) $path .= 'R'.$this->getId().'.pdf';
                        else if (file_exists($path.$this->__get('rnr').'.pdf')) $path .= $this->__get('rnr').'.pdf';
                        else $path .= $this->getId().'.pdf';

                    } else $path .= $this->getId().'.pdf';

                } else $path .= $this->getId().'.pdf';

            }
			
			\wpsg_ShopController::getShop()->protectDirectory(dirname($path));

			return $path;
			
		}

		/**
		 * @return \wpsg_order 
		 */
		public function getOrder() {

            return \wpsg_order::getInstance($this->getOrderIds()[0]);

		}

		public function getDate($format = null) {
			
			if ($format === null) return $this->__get('datum');
			else if ($format === true) return $this->getDate('d.m.Y H:i:s');
			else return date($format, strtotime($this->__get('datum')));
			
		}
		
		/**
		 * @param bool $bLink
		 * @param \wpsg_order_product $oOrderProduct
		 *
		 * @return string
		 */
		public function getNr($bLink = false, $oOrderProduct = null) {
			
			$strReturn = '';
			
			if ($bLink) {

				$strTitle = wpsg_translate(__('Erstellt am #1#', 'wpsg'), wpsg_formatTimestamp(strtotime($this->__get('datum'))));

				if ($this->getOrder()->getId() != '0') {

					$strReturn .= '<a title="'.$strTitle.'" class="link_invoice '.(($this->isStorno())?'bg_storno':'').' '.(($this->isInvoice())?'bg_invoice':'').'" href="';
					$strReturn .= wp_nonce_url(WPSG_URL_WP.'wp-admin/admin.php?page=wpsg-Order&action=ajax&noheader=1&mod=wpsg_mod_rechnungen&do=getRechnung&edit_id='.$this->getOrder()->getId().'&r_id='.$this->getId(), 'wpsg-mod_invoice-order_ajax-getRechnung-'.$this->getId());
					$strReturn .= '" target="_blank">';

				}

			}
			
			if ($this->isInvoice()) $strReturn .= 'R '.$this->__get('rnr');
			else if ($this->isStorno()) $strReturn .= 'RK '.$this->__get('gnr');
			
			if ($oOrderProduct !== null) {
				
				$invoice_data = json_decode($oOrderProduct->getMeta('wpsg_mod_rechnungen_data', false, 'null'), true);
				if ($invoice_data !== null) {
					
					foreach ($invoice_data as $id) {
						
						if ($id['invoice_id'] === $this->getId()) {
							
							$strReturn .= ' <span title="'.wpsg_translate(__('#1# Stück enthalten', 'wpsg'), $id['amount']).'">('.$id['amount'].')</span>';
							
						}
						
					}
					
				}
				
			}
			
			if ($bLink) {
				
				$strReturn .= '</a>';
				
			}
			
			return $strReturn;
			
		}

        public function delete() {

            $this->db->Query("DELETE FROM `".WPSG_TBL_ORDER_INVOICE."` WHERE `invoice_id` = '".intval($this->getId())."' ");
            $this->db->Query("DELETE FROM `".WPSG_TBL_RECHNUNGEN."` WHERE `id` = '".intval($this->getId())."' ");

            unlink($this->getFilePath());

        }

        /**
         * @param int $order_id
         * @return wpsg_invoice[]
         */
        public static function findByOrderId(int $order_id) {

            $r = [];

            foreach ($GLOBALS['wpsg_db']->fetchAssocField("SELECT `invoice_id` FROM `".WPSG_TBL_ORDER_INVOICE."` WHERE `order_id` = '".intval($order_id)."' ") as $invoice_id) {

                $r[] = wpsg_invoice::getInstance(intval($invoice_id));

            }

            return $r;

        }

	}